/* Main.c file generated by New Project wizard
 *
 * Project:			APA102_16F1825_2024
 * Created:   		4th March, 2024
 * Processor: 		PIC16F1825
 * Compiler:  		MPLAB XC8
 * Author:				Barry Phillips
 * Description:	Program to test the SPI function with APA102C LEDs using PIC16F1825 processor
 *							The program will toggle an array of  APA102C LEDs on and off at 1 second intervals
*							The colour is set to Blue and Global Power set to 0xFF 
 */

#include <xc.h>
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>

#define _XTAL_FREQ 8000000

#define num_led 8					     	//Set the number of LEDs in the string
														//APC102C power level is set with a 5-bit number preceded with "111"
														//Power = 0b111xxxxx
														//The datasheet calls this the "Global" setting
#define on 0xFF         						//LED Global Power level Full ON
#define off 0xE0        						//LED Global Power level Full OFF


// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection->INTOSC oscillator: I/O function on CLKIN pin
#pragma config WDTE = OFF    		// Watchdog Timer Enable->WDT disabled
#pragma config PWRTE = OFF    	// Power-up Timer Enable->PWRT disabled
#pragma config MCLRE = OFF    	// MCLR Pin Function Select->MCLR/VPP pin function is digital input
#pragma config CP = OFF    			// Flash Program Memory Code Protection->Program memory code protection is disabled
#pragma config CPD = OFF    		// Data Memory Code Protection->Data memory code protection is disabled
#pragma config BOREN = ON    		// Brown-out Reset Enable->Brown-out Reset enabled
#pragma config CLKOUTEN = OFF    // Clock Out Enable->CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin
#pragma config IESO = ON    			// Internal/External Switchover->Internal/External Switchover mode is enabled
#pragma config FCMEN = ON    		// Fail-Safe Clock Monitor Enable->Fail-Safe Clock Monitor is enabled

// CONFIG2
#pragma config WRT = OFF    		// Flash Memory Self-Write Protection->Write protection off
#pragma config PLLEN = OFF    		// PLL Enable->4x PLL disabled
#pragma config STVREN = ON    	// Stack Overflow/Underflow Reset Enable->Stack Overflow or Underflow will cause a Reset
#pragma config BORV = LO    		// Brown-out Reset Voltage Selection->Brown-out Reset Voltage (Vbor), low trip point selected.
#pragma config LVP = OFF    		// Low-Voltage Programming Enable->High-voltage on MCLR/VPP must be used for programming

void showpix(void);
void SPI_Send (uint8_t);

uint8_t blu = 255;							//Set Blue LED to full brightness
uint8_t red = 0;								//Set Green LED brightness off
uint8_t grn = 0; 								//Set Red LED brightness off
uint8_t pwr = 255;							//Variable for "Global" power level

void main(void)
 {
	/**
    LATx registers
    */
    LATA = 0x00;
    LATC = 0x00;

    /**
    TRISx registers
    */
    TRISA = 0x3F;
    TRISC = 0x32;

    /**
    ANSELx registers
    */
    ANSELC = 0x00;
    ANSELA = 0x13;

    /**
    WPUx registers
    */
    WPUA = 0x00;
    WPUC = 0x00;
    OPTION_REGbits.nWPUEN = 1;

    /**
    APFCONx registers
    */
    APFCON0 = 0x00;
    APFCON1 = 0x00;

	// SCS FOSC; SPLLEN disabled; IRCF 8MHz_HF; 
    OSCCON = 0x70;
    // TUN 0; 
    OSCTUNE = 0x00;
    // SBOREN disabled; 
    BORCON = 0x00;

	// WDTPS 1:65536; SWDTEN OFF; 
    WDTCON = 0x16;

	//SPI setup
    SSP1STAT = 0x40;
    SSP1CON1 = 0x00;
    SSP1ADD = 0x01;
    TRISCbits.TRISC0 = 0;
    SSP1CON1bits.SSPEN = 1;			//Enable SPI

       while (1)
    {
        showpix();
       LATCbits.LATC3 = 1;				//Heartbeat LED = ON
        __delay_ms(600);

        blu = 0;									//Turn Blue LED OFF
        showpix();
		blu = 255;								//Turn Blue LED ON
       LATCbits.LATC3 = 0;         		//Heartbeat LED = OFF
        __delay_ms(600);
    }
}
void showpix(void) {
    
        char b_cnt = 0;
        for (b_cnt = 0; b_cnt < 4; b_cnt++){    			//start frame
        SPI_Send(0);
    }

    for (b_cnt = 0; b_cnt < num_led; b_cnt++){  	//bgr data
		SPI_Send(pwr);
        SPI_Send(blu);
        SPI_Send(grn);
        SPI_Send(red);    
    }
    
    for (b_cnt = 0; b_cnt < 4; b_cnt++){    			//stop frame
        SPI_Send(255);
    }

}

void SPI_Send(uint8_t data) {							//Send one byte over the SPI bus
        SSP1BUF = data;
        while(!PIR1bits.SSP1IF);
        PIR1bits.SSP1IF = 0;
}

/**
 End of File
*/